<?php

namespace Teknisa\PED\Factories;

use Doctrine\ORM\EntityManager;
use Zeedhi\Framework\DTO\Response\Error;

class ReservaPedidoFactory {

    protected $connection;

    public function __construct(EntityManager $entityManager) {
        $this->connection    = $entityManager->getConnection();
        $this->entityManager = $entityManager;
    }

    public function vericaAlmoxarifadoVenda($filial, $almoxarifado = ' ' , $localestoque = ' ') {
        $sql = "SELECT
                    IDDISPCONPED
                FROM
                    LOCALESTOQUE
                WHERE
                    CDFILIAL         = :P_CDFILIAL
                    AND CDALMOXARIFE = :P_CDALMOXARIFE
                    AND CDLOCALESTOQ = NVL(:P_CDLOCALESTOQ, ' ')
        ";
        $params = array(
            'P_CDFILIAL' => $filial,
            'P_CDALMOXARIFE' => $almoxarifado,
            'P_CDLOCALESTOQ' => $localestoque
        );
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }

    public function retornaDadosFilial($cdfilial) {
        $sql = "SELECT
                    IDUTILALMOX,
                    IDUTILLCESTQ,
                    IDUTILLOTE
                FROM
                    PARAMFILIAL
                WHERE
                    CDFILIAL      = :P_CDFILIAL
        ";
        $params = array(
            'P_CDFILIAL' => $cdfilial,
        );
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }

    public function retornaAlmoxarifadoLoteProduto($cdfilial, $cdproduto) {
        $sql = "SELECT
                    CDALMOXPRCLI,
                    NVL(CDLOCALPRCLI, ' ') CDLOCALPRCLI
                FROM
                    PRODFILI
                WHERE
                    CDFILIAL      = :P_CDFILIAL
                    AND CDPRODUTO = :P_CDPRODUTO
        ";
        $params = array(
            'P_CDFILIAL' => $cdfilial,
            'P_CDPRODUTO' => $cdproduto
        );
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }

    public function vericaValidadeLote($cdfilial, $nrloteestq, $nrsublote, $cdproduto) {
        $sql = "SELECT
                    *
                FROM
                    LOTE
                WHERE
                    CDFILIAL       = :P_CDFILIAL
                    AND NRLOTEESTQ = :P_NRLOTEESTQ
                    AND NRSUBLOTE = :P_NRSUBLOTE
                    AND CDPRODUTO = :P_CDPRODUTO
        ";
        $params = array(
            'P_CDFILIAL'  => $cdfilial,
            'P_NRLOTEESTQ'=> $nrloteestq,
            'P_NRSUBLOTE' => $nrsublote,
            'P_CDPRODUTO' => $cdproduto
        );
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }

    public function retornaLoteProduto($cdfilial, $cdproduto) {
        $sql = "SELECT
                    *
                FROM
                    LOTE
                WHERE
                    CDFILIAL      = :P_CDFILIAL
                    AND CDPRODUTO = :P_CDPRODUTO
        ";
        $params = array(
            'P_CDFILIAL' => $cdfilial,
            'P_CDPRODUTO' => $cdproduto
        );
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }

    public function retornaLotesValidos($dtatual, $cdproduto, $cdfilial) {
        $sql = "SELECT
                    *
                FROM
                    LOTE
                WHERE
                    (:P_DTATUAL <= DTVALILOTE
                        OR DTVALILOTE     IS NULL)
                    AND CDPRODUTO      = :P_CDPRODUTO
                    AND CDFILIAL       = :P_CDFILIAL
                    AND NRLOTEESTQ IS NOT NULL
                ORDER BY
                    DTVALILOTE
        ";
        $params = array(
            'P_DTATUAL' => $dtatual,
            'P_CDPRODUTO' => $cdproduto,
            'P_CDFILIAL' => $cdfilial
        );
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }

        //-- não controla almoxarifado
    public function buscaEstoque($cdfilial, $nrpedido, $nritemped, $cdproduto, $cdalmoxarife = ' ', $cdlocalestoq = ' ') {

        //  Verifica estoque para um determinado item de pedido, verificando localestoque e almoxarifado;
        //  Nao verifica na prodfili se tem almoxarifado e local estoque caadastrado usa oque foi passado por parametro, caso nao seja passado e considerado ' ';

        $sql = "SELECT
                    CDFILIAL    , CDPRODUTO   , CDARVPROD   , CDALMOXARIFE,
                    CDLOCALESTOQ, CDFABRIC    , NMFABRIC    , DSALMOXARIFE,
                    DSLOCALESTOQ, IDDISPCONPED,
                    NRLOTEESTQ  , NRSUBLOTE   , QTTOTALESTOQ, NRPEDIDO    ,
                    NRITEMPED   , QTRESPRODPED, DTENTRLOTE  , DTVALILOTE  ,
                    SGUNIDADE   , CDPRODUTO CDPRODRES       , QTDISPONIVEL,
                    QTRESPRODARR, NRRESPRODPED, CDORDENA
                FROM
                   (SELECT
                        RE.CDFILIAL    , RE.CDPRODUTO   , PR.CDARVPROD   , RE.CDALMOXARIFE, LT.CDORDENALOTE,
                        RE.CDLOCALESTOQ, DECODE(RP.CDFABRIC,NULL,LT.CDFABRIC,RP.CDFABRIC ) CDFABRIC    ,
                        DECODE(RP.CDFABRIC,NULL,FA.NMFABRIC,F.NMFABRIC) NMFABRIC , AL.DSALMOXARIFE, LO.DSLOCALESTOQ,
                        LO.IDDISPCONPED, RE.NRLOTEESTQ  , RE.NRSUBLOTE   , RE.QTTOTALESTOQ, RP.NRPEDIDO,
                        RP.NRITEMPED   , RP.QTRESPRODPED, LT.DTENTRLOTE  , LT.DTVALILOTE  ,
                        PR.SGUNIDADE   , RE.CDPRODUTO CDPRODRES          , RE.QTDISPONIVEL,
                        RP.QTRESPRODARR, RP.NRRESPRODPED,
                        LT.CDORDENALOTE AS CDORDENA
                    FROM
                        PRODUTO PR, ALMOXARIFADO AL, LOCALESTOQUE LO,
                        LOTE    LT, RESERVAPED RE  , RESERPRODPED RP, FABRICANTE FA, FABRICANTE F
                    WHERE RP.NRPEDIDO       (+)    = :P_NRPEDIDO
                        AND RP.NRITEMPED    (+)    = :P_NRITEMPED
                        AND RE.CDALMOXARIFE        = NVL(:P_CDALMOXARIFE, RE.CDALMOXARIFE)
                        AND RE.CDLOCALESTOQ        = NVL(:P_CDLOCALESTOQ, RE.CDLOCALESTOQ)
                        AND RP.CDALMOXARIFE (+)    = RE.CDALMOXARIFE
                        AND RP.CDLOCALESTOQ (+)    = RE.CDLOCALESTOQ
                        AND RP.NRLOTEESTQ   (+)    = RE.NRLOTEESTQ
                        AND RP.NRSUBLOTE    (+)    = RE.NRSUBLOTE
                        AND RP.CDFILIAL     (+)    = RE.CDFILIAL
                        AND RP.CDPRODUTO    (+)    = RE.CDPRODUTO
                        AND RE.CDFILIAL            = :P_CDFILIAL
                        AND RE.CDPRODUTO           = :P_CDPRODUTO
                        AND RE.CDPRODUTO           = PR.CDPRODUTO
                        AND (RE.QTDISPONIVEL       > 0 OR RP.QTRESPRODPED > 0)
                        AND RE.CDFILIAL            = AL.CDFILIAL
                        AND RE.CDALMOXARIFE        = AL.CDALMOXARIFE
                        AND RE.CDFILIAL            = LO.CDFILIAL
                        AND RE.CDALMOXARIFE        = LO.CDALMOXARIFE
                        AND RE.CDLOCALESTOQ        = LO.CDLOCALESTOQ
                        AND RE.CDFILIAL            = LT.CDFILIAL   (+)
                        AND RE.NRLOTEESTQ          = LT.NRLOTEESTQ (+)
                        AND RE.NRSUBLOTE           = LT.NRSUBLOTE  (+)
                        AND RE.CDPRODUTO           = LT.CDPRODUTO  (+)
                        AND LT.CDFABRIC            = FA.CDFABRIC   (+)
                        AND RP.CDFABRIC            = F.CDFABRIC    (+)
                        AND (TRUNC(LT.DTVALILOTE) >= TRUNC(SYSDATE) OR (LT.DTVALILOTE IS NULL))
                    UNION
                    SELECT
                        RE.CDFILIAL    , RE.CDPRODUTO   , PR.CDARVPROD   , RE.CDALMOXARIFE, '' CDORDENALOTE ,
                        RE.CDLOCALESTOQ, RE.CDFABRIC    , FA.NMFABRIC    ,'' DSALMOXARIFE , '' DSLOCALESTOQ , '' IDDISPCONPED,
                        RE.NRLOTEESTQ  , RE.NRSUBLOTE   , 0 QTTOTALESTOQ , RE.NRPEDIDO    ,
                        RE.NRITEMPED   , RE.QTRESPRODPED, TO_DATE(NULL) DTENTRLOTE, TO_DATE(NULL) DTVALILOTE,
                        PR.SGUNIDADE   , RE.CDPRODUTO CDPRODRES          , 0 QTDISPONIVEL ,
                        RE.QTRESPRODARR, RE.NRRESPRODPED,
                        (RE.NRLOTEESTQ) AS CDORDENA
                    FROM
                         PRODUTO PR, RESERPRODPED RE, FABRICANTE FA
                    WHERE RE.CDFILIAL     = :P_CDFILIAL
                        AND RE.CDPRODUTO    = :P_CDPRODUTO
                        AND RE.NRPEDIDO     = :P_NRPEDIDO
                        AND RE.NRITEMPED    = :P_NRITEMPED
                        AND RE.CDPRODUTO    = PR.CDPRODUTO
                        AND RE.QTRESPRODPED > 0
                        AND RE.CDALMOXARIFE IS NULL
                        AND RE.CDLOCALESTOQ IS NULL
                        AND RE.CDFABRIC     =  FA.CDFABRIC (+)
                    ) A
                ORDER BY
                   CDFILIAL , CDARVPROD   , DTVALILOTE, CDORDENALOTE, CDORDENA, DTENTRLOTE, NRLOTEESTQ,
                   NRSUBLOTE, CDALMOXARIFE, CDLOCALESTOQ
        ";
        $params = array(
            'P_NRPEDIDO'=>$nrpedido,
            'P_NRITEMPED'=>$nritemped,
            'P_CDALMOXARIFE'=>$cdalmoxarife,
            'P_CDLOCALESTOQ'=>$cdlocalestoq,
            'P_CDFILIAL'=>$cdfilial,
            'P_CDPRODUTO'=>$cdproduto
        );
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }

        //-- controla almoxarifado
    public function buscaEstoqueAlmox ($cdfilial, $nrpedido, $nritemped, $cdproduto, $cdalmoxarife = ' ', $cdlocalestoq = ' ') {
        //  Verifica estoque para um determinado item de pedido, verificando localestoque e almoxarifado;
        //  Verifica na prodfili se tem almoxarifado e local estoque caadastrado e usa oque foi passado por parametro, caso nao seja passado e considerado ' ';
        $sql = "SELECT
                   CDFILIAL    , CDPRODUTO   , CDARVPROD   , CDALMOXARIFE,
                   CDLOCALESTOQ, CDFABRIC    , NMFABRIC    , DSALMOXARIFE,
                   DSLOCALESTOQ, IDDISPCONPED,
                   NRLOTEESTQ  , NRSUBLOTE   , QTTOTALESTOQ, NRPEDIDO,
                   NRITEMPED   , QTRESPRODPED, DTENTRLOTE  , DTVALILOTE,
                   SGUNIDADE   , CDPRODUTO CDPRODRES          , QTDISPONIVEL,
                   QTRESPRODARR, NRRESPRODPED, CDORDENA
                FROM
                   (SELECT
                       RE.CDFILIAL    , RE.CDPRODUTO   , PR.CDARVPROD   , RE.CDALMOXARIFE, LT.CDORDENALOTE,
                       RE.CDLOCALESTOQ, DECODE(RP.CDFABRIC,NULL,LT.CDFABRIC,RP.CDFABRIC ) CDFABRIC    ,
                       DECODE(RP.CDFABRIC,NULL,FA.NMFABRIC,F.NMFABRIC) NMFABRIC    , AL.DSALMOXARIFE, LO.DSLOCALESTOQ,
                       LO.IDDISPCONPED, RE.NRLOTEESTQ  , RE.NRSUBLOTE   , RE.QTTOTALESTOQ, RP.NRPEDIDO,
                       RP.NRITEMPED   , RP.QTRESPRODPED, LT.DTENTRLOTE  , LT.DTVALILOTE,
                       PR.SGUNIDADE   , RE.CDPRODUTO CDPRODRES          , RE.QTDISPONIVEL,
                       RP.QTRESPRODARR, RP.NRRESPRODPED,
                       LT.CDORDENALOTE AS CDORDENA
                    FROM
                       PRODUTO PR, ALMOXARIFADO AL, LOCALESTOQUE LO,
                       LOTE    LT, RESERVAPED   RE, RESERPRODPED RP, FABRICANTE FA, FABRICANTE F
                    WHERE RP.NRPEDIDO     (+)    = :P_NRPEDIDO
                      AND RP.NRITEMPED    (+)    = :P_NRITEMPED
                      AND RE.CDALMOXARIFE        = NVL(:P_CDALMOXARIFE, RE.CDALMOXARIFE)
                      AND RE.CDLOCALESTOQ        = NVL(:P_CDLOCALESTOQ, RE.CDLOCALESTOQ)
                      AND RP.CDALMOXARIFE (+)    = RE.CDALMOXARIFE
                      AND RP.CDLOCALESTOQ (+)    = RE.CDLOCALESTOQ
                      AND RP.NRLOTEESTQ   (+)    = RE.NRLOTEESTQ
                      AND RP.NRSUBLOTE    (+)    = RE.NRSUBLOTE
                      AND RP.CDFILIAL     (+)    = RE.CDFILIAL
                      AND RP.CDPRODUTO    (+)    = RE.CDPRODUTO
                      AND RE.CDFILIAL            = :P_CDFILIAL
                      AND RE.CDPRODUTO           = :P_CDPRODUTO
                      AND RE.CDPRODUTO           = PR.CDPRODUTO
                      AND (RE.QTDISPONIVEL       > 0 OR RP.QTRESPRODPED > 0)
                      AND RE.CDFILIAL            = AL.CDFILIAL
                      AND RE.CDALMOXARIFE        = AL.CDALMOXARIFE
                      AND RE.CDFILIAL            = LO.CDFILIAL
                      AND RE.CDALMOXARIFE        = LO.CDALMOXARIFE
                      AND RE.CDLOCALESTOQ        = LO.CDLOCALESTOQ
                      AND RE.CDFILIAL            = LT.CDFILIAL   (+)
                      AND RE.NRLOTEESTQ          = LT.NRLOTEESTQ (+)
                      AND RE.NRSUBLOTE           = LT.NRSUBLOTE  (+)
                      AND RE.CDPRODUTO           = LT.CDPRODUTO  (+)
                      AND LT.CDFABRIC            = FA.CDFABRIC   (+)
                      AND RP.CDFABRIC            = F.CDFABRIC    (+)
                      AND LO.IDDISPCONPED        = 'S'
                      AND (TRUNC(LT.DTVALILOTE) >= TRUNC(SYSDATE) OR (LT.DTVALILOTE IS NULL))
                      AND (LO.CDLOCALESTOQ = (SELECT NVL(CDLOCALPRCLI, ' ')
                                                FROM PRODFILI PF
                                               WHERE PF.CDFILIAL     = :P_CDFILIAL
                                                 AND PF.CDPRODUTO    = :P_CDPRODUTO
                                                 AND PF.CDALMOXPRCLI = LO.CDALMOXARIFE))
                    UNION
                    SELECT
                       RE.CDFILIAL    , RE.CDPRODUTO   , PR.CDARVPROD   , RE.CDALMOXARIFE, '' CDORDENALOTE ,
                       RE.CDLOCALESTOQ, RE.CDFABRIC    , FA.NMFABRIC    ,'' DSALMOXARIFE , '' DSLOCALESTOQ , '' IDDISPCONPED,
                       RE.NRLOTEESTQ  , RE.NRSUBLOTE   , 0 QTTOTALESTOQ , RE.NRPEDIDO    ,
                       RE.NRITEMPED   , RE.QTRESPRODPED, TO_DATE(NULL) DTENTRLOTE, TO_DATE(NULL) DTVALILOTE,
                       PR.SGUNIDADE   , RE.CDPRODUTO CDPRODRES          , 0 QTDISPONIVEL,
                       RE.QTRESPRODARR, RE.NRRESPRODPED,
                       (RE.NRLOTEESTQ) AS CDORDENA
                    FROM
                       PRODUTO PR, RESERPRODPED RE, FABRICANTE FA
                    WHERE RE.CDFILIAL     = :P_CDFILIAL
                      AND RE.CDPRODUTO    = :P_CDPRODUTO
                      AND RE.NRPEDIDO     = :P_NRPEDIDO
                      AND RE.NRITEMPED    = :P_NRITEMPED
                      AND RE.CDPRODUTO    = PR.CDPRODUTO
                      AND RE.QTRESPRODPED > 0
                      AND RE.CDALMOXARIFE IS NULL
                      AND RE.CDLOCALESTOQ IS NULL
                      AND RE.CDFABRIC     =  FA.CDFABRIC (+)
                   ) A
                ORDER BY
                   CDFILIAL, CDARVPROD, DTVALILOTE,  CDORDENALOTE, CDORDENA, DTENTRLOTE, NRLOTEESTQ,
                   NRSUBLOTE, CDALMOXARIFE, CDLOCALESTOQ
        ";
        $params = array(
            'P_NRPEDIDO'=>$nrpedido,
            'P_NRITEMPED'=>$nritemped,
            'P_CDALMOXARIFE'=>$cdalmoxarife,
            'P_CDLOCALESTOQ'=>$cdlocalestoq,
            'P_CDFILIAL'=>$cdfilial,
            'P_CDPRODUTO'=>$cdproduto
        );
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
}